<?php

class Gmap_Direx_Route {

  var $valid;
  var $rid;
  var $uid;  
  var $hash;
  var $locale;
  var $name;
  var $mapcenterlat;
  var $mapcenterlng;
  var $copyright;
  var $summary;
  var $distance;
  var $duration;
  var $geocode;
  var $steps;
  var $waypointline;
  var $polyline;
  
  function Gmap_Direx_Route($route) {
    $this->valid = FALSE;
    if (is_numeric($route)) {
      $this->valid = $this->load($route);
    }
    else {
      $this->valid = $this->init_by_string($route);
    }
  }

  function init_by_string($dir) {
    if (!isset($dir) || empty($dir)) {
      return FALSE;
    }
    global $user;
    $this->rid = 0;
    $this->uid = $user->uid;
    $dir = str_replace('#-#', '"', $dir);
    $dir = json_decode($dir);
    $this->locale = $dir->locale;
    $this->mapcenterlat = $dir->center->y;
    $this->mapcenterlng = $dir->center->x;
    $this->copyright = $dir->copyrighthtml;
    $this->summary = $dir->summaryhtml;
    $this->distance = $dir->distance->meters;
    $this->duration = $dir->duration->seconds;
    $this->geocode = get_object_vars($dir->geocode);
    $this->steps = array();
    $this->waypointline = array();
    $key = array($this->uid, $this->locale);
    $sr = 1;
    foreach ($dir->route as $route) {
      foreach ($route as $step) {
        $step->subroute = $sr;
        $this->steps[] = $step;
        $this->waypointline[] = array($step->LatLng->y, $step->LatLng->x);
        $key[] = $sr .','. $step->LatLng->y .','. $step->LatLng->x;
      }
      $sr++;
    }
    $this->hash = md5(implode(',', $key));
    $this->polyline = array();
    foreach ($dir->polyline as $step) {
      $this->polyline[] = array($step->Lat, $step->Lng);
    }
    $start = $this->start_waypoint();
    $stop = $this->end_waypoint();
    $vias = $this->via();
    if (sizeof($vias)) {
      $via = ' ('. t('via @via', array('@via' => implode(', ', $vias),)) .')';
    }
    $this->name = t('From "@from" to "@to" @via', array('@from' => $start, '@to' => $stop, '@via' => $via,));
    return TRUE;
  }

  function via() {
    $waypoints = $this->geocode;
    $wpn = sizeof($waypoints);
    $vias = array();
    for ($i = 1; $i < $wpn - 1; $i++) {
      $vias[] = strip_tags($waypoints{$i}->address);
    }
    return $vias;
  }

  function start_waypoint() {
    return strip_tags($this->geocode{0}->address);
  }

  function end_waypoint() {
    $waypoints = $this->geocode;
    $wpn = sizeof($waypoints);
    return strip_tags($waypoints{$wpn-1}->address);
  }

  function explanation() {
    $output = '<ul>';
    foreach ($this->steps as $step) {
      $output .= '<li>'. $step->Description .'</li>';
    }
    $output .= '</ul>';
    return $output;
  }

  function map() {
    $shape = array(
        'type' => "line",
        'points' => $this->polyline,
        'style' => array("ff0000", 5, 80, "00ff00", 60),
      );
    $map_array = array(
      'id' => "my-map",
      'width' => "100%",
      'height' => "600px",
      'latitude' => $this->mapcenterlat,
      'longitude' => $this->mapcenterlng,
      'zoom' => 10,
      'maptype' => "Map",
      'controltype' => "Small",
      'shapes' => array($shape,),
    );
    return theme('gmap', $map_array);
  }

  function already_exists() {
    $sql = "SELECT rid FROM {gmap_direx_route} WHERE uid = %d AND hash = '%s'";
    $rid = db_result(db_query($sql, $this->uid, $this->hash));
    return $rid;
  }

  function load($rid) {
    $r = db_fetch_object(db_query("SELECT * FROM {gmap_direx_route} WHERE rid = %d", $rid));
    if (!$r) {
      return FALSE;
    }
    $s = db_query("SELECT * FROM {gmap_direx_route_step} WHERE rid = %d ORDER BY sid", $rid);
    $p = db_query("SELECT * FROM {gmap_direx_route_polyline} WHERE rid = %d ORDER BY sid", $rid);
    $this->uid = $r->uid;
    $this->hash = $r->hash;
    $this->name = $r->name;
    $this->locale = $r->locale;
    $this->mapcenterlng = $r->clng;
    $this->mapcenterlat = $r->clat;
    $this->copyright = $r->copyright;
    $this->summary = $r->summary;
    $this->duration = $r->duration;
    $this->distance = $r->distance;
    $this->geocode = unserialize($r->geocode);
    $this->steps = array();
    $this->waypointline = array();
    $this->polyline = array();
    while ($step = db_fetch_object($s)) {
      $st = new stdClass();
      $st->subroute = $step->srid;
      $st->LatLng = new stdClass();
      $st->LatLng->x = $step->lng;
      $st->LatLng->y = $step->lat;
      $st->Description = htmlspecialchars_decode($step->description);
      $st->Duration = new stdClass();
      $st->Duration->seconds = $step->duration;
      $st->Distance = new stdClass();
      $st->Distance->meters = $step->distance;
      $this->steps[] = $st;
      $this->waypointline[] = array($step->lat, $step->lng);
    }
    while ($step = db_fetch_object($p)) {
      $this->polyline[] = array($step->lat, $step->lng);
    }
    return TRUE;
  }

  function save() {
    db_query("DELETE FROM {gmap_direx_route} WHERE rid=%d", $this->rid);
    db_query("DELETE FROM {gmap_direx_route_step} WHERE rid=%d", $this->rid);
    db_query("DELETE FROM {gmap_direx_route_polyline} WHERE rid=%d", $this->rid);
    db_query("INSERT INTO {gmap_direx_route}
        (uid, hash, name, locale, clng, clat, copyright, summary, duration, distance, geocode)
        VALUES (%d, '%s', '%s', '%s', '%s', '%s', '%s', '%s', %d, %d, '%s')",
        $this->uid, $this->hash, $this->name, $this->locale, $this->mapcenterlng, $this->mapcenterlat, $this->copyright, $this->summary, $this->duration, $this->distance, serialize($this->geocode));
    $this->rid = db_last_insert_id('gmap_direx_route', 'rid');
    $steps = array();
    $i = 0;
    foreach ($this->steps as $step) {
      $i++;
      $steps[] = '('. check_plain($this->rid) .','.
                      check_plain($step->subroute) .','.
                      check_plain($i) .','.
                      "'". check_plain($step->LatLng->x)  ."'".','.
                      "'". check_plain($step->LatLng->y)  ."'".','.
                      "'". check_plain($step->Description)  ."'".','.
                      check_plain($step->Duration->seconds) .','.
                      check_plain($step->Distance->meters) .')';
    }
    db_query("INSERT INTO {gmap_direx_route_step}
        (rid, srid, sid, lng, lat, description, duration, distance)
        VALUES ". implode(',', $steps));
    $steps = array();
    $i = 0;
    foreach ($this->polyline as $step) {
      $i++;
      $steps[] = '('. check_plain($this->rid) .','.
                      check_plain($i) .','.
                      "'". check_plain($step[1]) ."'".','.
                      "'". check_plain($step[0]) ."'".')';
    }
    db_query("INSERT INTO {gmap_direx_route_polyline}
        (rid, sid, lng, lat)
        VALUES ". implode(',', $steps));
  }

}
